@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2020..2021 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2020..2025 @BABOLO http://www.babolo.ru/
  * PKG = babolo-libmake
  * All rights reserved.
  *
@@ -25,8 +25,8 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2020..2021 @BABOLO http://www.babolo.ru/\n"
-#ident "@(#) $Id: blinread.c,v 1.55 2021/10/25 15:04:44 cuhegh Exp $\n"
+#ident "@(#) Copyright (C)2020..2025 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: blinread.c,v 1.61 2025/02/10 01:08:50 babolo Exp $"
 
 #define Bpars_COMPAT  4
 #define MULAR_COMPAT  0
@@ -56,6 +56,7 @@
 
 static const size_t msz[] = {1024, 512, 512};
 
+__attribute__((__noreturn__))
 static void
 /*****************************************************************************************************/
 usage(int ex) {                                                                                  /****
@@ -66,8 +67,8 @@ usage(int ex) {
              "   -c TEXT         read from TEXT\n"
              "   -Q              give information about tracefile\n"
              "   -l MASK         filter by message level\n"
-             "   -d NUM[-NUM]    filter by message data,\n"
-             "                     where NUM is <YYYY-MM-DD[_hh:mm:ss[.nsec]]>\n"
+             "   -d DATE[-DATE]  filter by message date,\n"
+             "                     where DATE is <YYYY-MM-DD[_hh:mm:ss[.nsec]]>\n"
              "   -t NUM[-NUM]    filter by relative message time, where NUM is <sec[.nsec]>\n"
              "   -i NUM          filter by pid\n"
              "   -r TEXT         filter by program name\n"
@@ -101,29 +102,31 @@ usage(int ex) {
 
 int
 main(int argc, char *argv[]) {
+#   define blin_internal_flags (cfg->flags & BLIN_MASK)
 #   define isJX(EXCODE, FORMAT, ...) \
         do { ifBLIN_QX0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
 #   define isJW(EXCODE, FORMAT, ...) \
         do { ifBLIN_QW0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
-#   define blin_internal_flags (cfg->flags & BLIN_MASK)
-    const char *const opts = "c:Dd:eIi:Ll:Nn:p:QqRr:Ss:Tt:Uu:vx?=:";
-    babolo_opts      *bos = NULL;              /*             */
-    char              fl;                      /*                                       */
-    const char       *optargs;                 /*                              */
-    int               ex = EX_OK;              /*                                */
     BLINr_fraze      *curfraze;                /*                           */
+    const char       *optargs;                 /*                              */
+    const char       *tname;                   /*  tracfile                              */
     const char       *cname = NULL;            /*                             */
     mife_descriptor  *cfile = NULL;            /*                                 */
     char             *cchar = NULL;            /*                       */
-    babolo_parm      *bp;                      /*   -                          */
-    const char       *tname;                   /*  tracfile                              */
     BLIN_trace       *twind;                   /*   tracfile                           */
-    u_int64_t         m;                       /* ޣ,              */
+    const char *const opts  = "c:Dd:eIi:Ll:Nn:p:QqRr:Ss:Tt:Uu:vx?=:";
     BLINr_cfg        *cfg;
+    babolo_opts      *bos   = NULL;            /*             */
+    char              fl;                      /*                                       */
+    int               ex    = EX_OK;           /*                                */
+    babolo_parm      *bp;                      /*   -                          */
+    u_int64_t         m;                       /* ޣ,              */
+    char             *p;
 
     if  (!(cfg = calloc(1, sizeof(BLINr_cfg)))) isJW(EX_OSERR, "cfg alloc");
-    blin_ctl(BLIN_CTL_FLAG | BLIN_CTL_FSET, (cfg->flags = BLIN_BIT0));
-    cfg->flags = BLINREAD_P_H | BLINREAD_P_L | BLINREAD_P_D | BLINREAD_P_I;
+    cfg->flags = BLIN_BIT0;
+    blin_ctl(BLIN_CTL_FLAG | BLIN_CTL_FSET, cfg->flags);
+    cfg->flags |= BLINREAD_P_H | BLINREAD_P_L | BLINREAD_P_D | BLINREAD_P_I;
     cfg->flags |= BLINREAD_P_R | BLINREAD_P_S | BLINREAD_P_U | BLINREAD_P_N;
     if  (!(curfraze = cfg->filt = malloc(sizeof(BLINr_fraze)))) isJW(EX_OSERR, "curfraze malloc");
     cfg->time = mular_create(MULAR_OBJ | MULAR_STRI, 3, sizeof(BLINr_time), msz);
@@ -141,9 +144,9 @@ main(int argc, char *argv[]) {
     if  (!(cfg->tfile = mife_init(MIFE_FULL))) isJW(EX_OSERR, "mife_init");
     if  (!(bos = babolo_openopts(Bpars_SYME | Bpars_ESYM, 'Z'))) isJW(EX_OSERR, "babolo_openopts");
     if  (babolo_setopts(bos, Bpars_NONU, argc, argv, opts))    isJW(EX_OSERR, "babolo_setopts");
-    *curfraze = (BLINr_fraze){NULL, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, 0, 0, BLINREAD_X};
+    bzero(curfraze, sizeof(BLINr_fraze));
     cfg->prog->tofree = cfg->file->tofree = cfg->func->tofree = free; /* free() */
-    while(!!(fl = babolo_getopts(bos))) {
+    while(!!(fl = (char)babolo_getopts(bos))) {
         ifBLIN_QX2("flag %02X -%c", fl, fl);
         switch(fl) {
         case 'c':
@@ -169,8 +172,7 @@ main(int argc, char *argv[]) {
 
                 if  (!time) isJW(EX_OSERR, "flag %c mular_insert %u", fl, curfraze->time.to - 1);
                 if  (!(optargs = babolo_getoptsarg(bos)))    isJW(EX_DATAERR, "flag %c args", fl);
-                time->abs = 1;
-                if  (!!tim2i(cfg, time, optargs)) isJX(EX_USAGE, "flag %c tim2i", fl);
+                if  (!!timeabs2i(cfg, time, optargs)) isJX(EX_USAGE, "flag %c timeabs2i", fl);
                 errno = 0;
             }
             break;
@@ -186,7 +188,7 @@ main(int argc, char *argv[]) {
             curfraze->next->line.from = curfraze->next->line.to = curfraze->line.to;
             curfraze->next->level = 0x00;
             curfraze->next->levfl = 0;
-            curfraze->next->flags = 0x00;
+            curfraze->next->flags = BLINREAD_X;
             curfraze = curfraze->next;
             break;
         case 'I':
@@ -196,7 +198,7 @@ main(int argc, char *argv[]) {
                 u_int32_t *pid = mular_insert(cfg->pid, curfraze->pid.to++);
 
                 if  (!pid) isJW(EX_OSERR, "flag %c mular_insert %u", fl, curfraze->pid.to - 1);
-                *pid = babolo_getoptunum(bos, 10);
+                *pid = (u_int32_t)babolo_getoptunum(bos, 10);
                 if  (errno) isJW(EX_USAGE, "flag %c wrong arg", fl);
             }
             break;
@@ -205,7 +207,7 @@ main(int argc, char *argv[]) {
             ifBLIN_QX2("-flag %c", fl);
             break;
         case 'l': {
-                u_int8_t optunum = babolo_getoptunum(bos, 0);
+                u_int8_t optunum = (u_int8_t)babolo_getoptunum(bos, 0);
 
                 if  (errno) isJW(EX_USAGE, "flag %c wrong arg", fl);
                 curfraze->level |= optunum;
@@ -220,11 +222,11 @@ main(int argc, char *argv[]) {
 
                 if  (!line) isJW(EX_OSERR, "flag %c mular_insert %u", fl, curfraze->line.to - 1);
                 if  (!(optargs = babolo_getoptsarg(bos))) isJW(EX_USAGE, "flag %c args", fl);
-                line->from = strtoul(optargs, (char**) &optargs, 0);
-                if  (*optargs == '-') {
+                line->from = (u_int32_t)strtoul(optargs, &p, 0);
+                if  (*p == '-') {
                     line->nrng = 0;
-                    optargs++;
-                    line->to = strtoul(optargs, NULL, 0);
+                    p++;
+                    line->to = (u_int32_t)strtoul(p, NULL, 0);
                 } else {
                     line->nrng = 1;
             }   }
@@ -244,10 +246,10 @@ main(int argc, char *argv[]) {
                     case 'u': m |= BLINREAD_P_U; break;
                     case 'n': m |= BLINREAD_P_N; break;
                     case 'e': m |= BLINREAD_P_E; break;
-                    case 'x': m = 0; i = strlen(optargs); break;
+                    case 'x': m = 0; i = (u_int8_t)strlen(optargs); break;
                     default: isJX(EX_USAGE, "flag %c wrong arg", fl);
                 }   }
-                cfg->flags = (cfg->flags & ~BLINREAD_PMASC) | m;
+                cfg->flags = (cfg->flags & ~BLINREAD_PMASC) | (BLIN_flag)m;
             }
             break;
         case 'Q':
@@ -285,8 +287,7 @@ main(int argc, char *argv[]) {
 
                 if  (!time) isJW(EX_OSERR, "flag %c mular_insert %u", fl, curfraze->time.to - 1);
                 if  (!(optargs = babolo_getoptsarg(bos))) isJW(EX_USAGE, "flag %c args", fl);
-                time->abs = 0;
-                if  (!!tim2i(cfg, time, optargs)) isJX(EX_USAGE, "flag %c tim2i", fl);
+                if  (!!timerel2i(cfg, time, optargs)) isJX(EX_USAGE, "flag %c timerel2i", fl);
                 errno = 0;
             }
             break;
@@ -312,7 +313,7 @@ main(int argc, char *argv[]) {
             curfraze->line.from = curfraze->line.to;
             curfraze->level     = 0x00;
             curfraze->levfl     = 0;
-            curfraze->flags     = BLINREAD_X;
+            curfraze->flags     = 0x00;
             break;
         case '?':
             usage(EX_OK);
@@ -334,7 +335,7 @@ main(int argc, char *argv[]) {
     if  (!(tname = babolo_getargs(bos))) isJX(EX_USAGE, "no tracefile");
     if  (0 > mife_ctlfile(cfg->tfile, tname)) isJX(EX_USAGE, "no such tracefile");
     twind = BLINr_next_wind(cfg);
-    if  (!twind) isJW(EX_IOERR, "tracefile window %"BLIN_U, cfg->tfile->ofpoint);
+    if  (!twind) isJW(EX_IOERR, "tracefile window %"BLIN_O"u", cfg->tfile->ofpoint);
     if  (cfg->flags & BLINREAD_P_H) {
         if  (cfg->flags & BLINREAD_P_I) printf("pid ");
         if  (cfg->flags & BLINREAD_P_T) {
