@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2020..2021 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2020..2025 @BABOLO http://www.babolo.ru/
  * PKG = babolo-libmake
  * All rights reserved.
  *
@@ -25,8 +25,8 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2020..2021 @BABOLO http://www.babolo.ru/\n"
-#ident "@(#) $Id: over.c,v 1.10 2021/10/25 15:04:44 cuhegh Exp $\n"
+#ident "@(#) Copyright (C)2020..2025 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: over.c,v 1.21 2025/04/22 03:13:40 babolo Exp $"
 
 #define Bpars_COMPAT  4
 #define MULAR_COMPAT  0
@@ -36,23 +36,27 @@
 
 #include <sys/types.h>
 #include <sys/event.h>
-#include <sys/time.h>
 #include <sys/file.h>
 #include <sysexits.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stddef.h>
-#include <malloc_np.h>
 #include <limits.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
+#include <time.h>
 #include <err.h>
 #include "../BLINflag.h"
 #include <babolo/parser.h>
 #include <multilar.h>
 #include <mife.h>
+
+#ifndef BLIN_OSLinux
+# include <malloc_np.h>
+#endif
+
 #include "blinread.h"
 
 static const size_t msz[] = {1024, 512, 512};
@@ -62,7 +66,7 @@ compare_sun(BLINr_qbuf_sun *sun, BLIN_trace *twind) {
     int8_t com;
 
     if  (sun->filesz == twind->filesz) {
-        com = strncmp(*sun->file, &twind->string[twind->progsz], twind->filesz);
+        com = (int8_t)strncmp(*sun->file, &twind->string[twind->progsz], twind->filesz);
         if  (com != 0) {
             com *= 3;
             return(com);
@@ -73,7 +77,7 @@ compare_sun(BLINr_qbuf_sun *sun, BLIN_trace *twind) {
         return(-3);
     }
     if  (sun->funcsz == twind->funcsz) {
-        com = strncmp(*sun->func, &twind->string[twind->progsz + twind->filesz], twind->funcsz);
+        com = (int8_t)strncmp(*sun->func, &twind->string[twind->progsz + twind->filesz], twind->funcsz);
         if  (com != 0) {
             com *= 2;
             return(com);
@@ -91,70 +95,68 @@ compare_sun(BLINr_qbuf_sun *sun, BLIN_trace *twind) {
         return(-1);
 }   }
 
-void
+static void
 sunfree(void *fre) {
     BLINr_qbuf_sun *sun = fre;
-    if  (!*sun->file) {
+    if  (!!*sun->file) {
         free(*sun->file);
         *sun->file = NULL;
     }
-    if  (!*sun->func) {
+    if  (!!*sun->func) {
         free(*sun->func);
         *sun->func = NULL;
 }   }
 
 static int
 srch_pid(BLINr_cfg *cfg, BLIN_trace *twind) {
-#   define isJX(EXCODE, FORMAT, ...) \
-        do { ifBLIN_QX0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
-#   define isJW(EXCODE, FORMAT, ...) \
-        do { ifBLIN_QW0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
 #   define blin_internal_flags (cfg->flags & BLIN_MASK)
-    int               ex = EX_OK;                /*   */
     mular_descriptor *mular = cfg->qbuf->pid;    /*    */
-    u_int32_t        *pid = NULL;
     size_t            first = 0;
-    size_t            mid = 0;
-    size_t            last = 0;
+    size_t            last  = 0;
+    u_int32_t        *pid   = NULL;
+    size_t            mid   = 0;
+    int               ex    = EX_OK;             /*   */
 
     ifBLIN_QX2("+srch_pid");
     last = MULAR_NEXT(mular);
     if  (last == 0) goto put;
-    ifBLIN_QX2("last=%zd", last);
+    ifBLIN_QX2("last=%zu", last);
     while(first != last) {
         mid = (first + last) / 2;
-        ifBLIN_QX4("mid=%zd", mid);
+        ifBLIN_QX4("mid=%zu", mid);
         pid = mular_getix(mular, mid);
-        if  (!pid) isJW(1, "getix %zd", mid);
-        ifBLIN_QX4("getix(%zd) %u", mid, *pid);
-        if  (*pid == twind->pid) {
+        if  (!pid) {
+            ifBLIN_QW0("getix %zu", mid);
+            ex = 1;
             goto out;
-        } else if (*pid > twind->pid) {
+        }
+        ifBLIN_QX4("getix(%zu) %u", mid, *pid);
+        if  (*pid == (u_int32_t)twind->pid) {
+            goto out;
+        } else if (*pid > (u_int32_t)twind->pid) {
             last = mid;
         } else {
             if (first == mid) mid = last;
             first = mid;
         }
-        ifBLIN_QX4("first=%zd last=%zd", first, last);
+        ifBLIN_QX4("first=%zu last=%zu", first, last);
     }
 put:
     pid = mular_insert(mular, mid);
-    if  (!pid) isJW(EX_OSERR, "insert %zd", mid);
-    *pid = twind->pid;
+    if  (!pid) {
+        ifBLIN_QW0("insert %zu", mid);
+        ex = EX_OSERR;
+        goto out;
+    }
+    *pid = (u_int32_t)twind->pid;
 out:
     ifBLIN_QX2("-srch_pid %zu %u", mid, *pid);
     return(ex);
-#   undef isJX
-#   undef isJW
 #   undef blin_internal_flags
 }
 
 static int
 srch_prog(BLINr_cfg *cfg, BLIN_trace *twind) {
-#   define isJX(EXCODE, FORMAT, ...) \
-        do { ifBLIN_QX0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
-#   define isJW(EXCODE, FORMAT, ...) \
-        do { ifBLIN_QW0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
 #   define blin_internal_flags (cfg->flags & BLIN_MASK)
     int               ex = EX_OK;                /*   */
     mular_descriptor *mular = cfg->qbuf->prog;   /*    */
@@ -167,15 +169,19 @@ srch_prog(BLINr_cfg *cfg, BLIN_trace *twind) {
     ifBLIN_QX2("+srch_prog");
     last = MULAR_NEXT(mular);
     if  (last == 0) goto put;
-    ifBLIN_QX2("last=%zd", last);
+    ifBLIN_QX2("last=%zu", last);
     while(first != last) {
         mid = (first + last) / 2;
-        ifBLIN_QX4("mid=%zd", mid);
+        ifBLIN_QX4("mid=%zu", mid);
         prog = mular_getix(mular, mid);
-        if  (!prog) isJW(1, "getix %zd", mid);
-        ifBLIN_QX4("getix(%zd) %s", mid, *prog);
+        if  (!prog) {
+            ifBLIN_QW0("getix %zu", mid);
+            ex = 1;
+            goto out;
+        }
+        ifBLIN_QX4("getix(%zu) %s", mid, *prog);
         if  (strlen(*prog) == twind->progsz) {
-            com = strncmp(*prog, &twind->string[0], twind->progsz);
+            com = (int8_t)strncmp(*prog, &twind->string[0], twind->progsz);
         } else if (strlen(*prog) > twind->progsz){
             com = 1;
         } else {
@@ -189,11 +195,15 @@ srch_prog(BLINr_cfg *cfg, BLIN_trace *twind) {
             if (first == mid) mid = last;
             first = mid;
         }
-        ifBLIN_QX4("first=%zd last=%zd", first, last);
+        ifBLIN_QX4("first=%zu last=%zu", first, last);
     }
 put:
     prog = mular_insert(mular, mid);
-    if  (!prog) isJW(EX_OSERR, "insert %zd", mid);
+    if  (!prog) {
+        ifBLIN_QW0("insert %zu", mid);
+        ex = EX_OSERR;
+        goto out;
+    }
     ifBLIN_QX2("now strndup prog");
     ifBLIN_QX4( "strndup %u %.*s"
               , twind->progsz
@@ -202,22 +212,20 @@ put:
               )
     ;
     *prog = strndup(&twind->string[0], twind->progsz);
-    if  (!*prog) isJW(EX_OSERR, "strndup");
+    if  (!*prog) {
+        ifBLIN_QW0("strndup");
+        ex = EX_OSERR;
+        goto out;
+    }
     ifBLIN_QX4("*prog = %s", *prog);
 out:
     ifBLIN_QX2("-srch_prog %zu %s", mid, *prog);
     return(ex);
-#   undef isJX
-#   undef isJW
 #   undef blin_internal_flags
 }
 
 static int
 srch_sun(BLINr_cfg *cfg, BLIN_trace *twind) {
-#   define isJX(EXCODE, FORMAT, ...) \
-        do { ifBLIN_QX0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
-#   define isJW(EXCODE, FORMAT, ...) \
-        do { ifBLIN_QW0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
 #   define blin_internal_flags (cfg->flags & BLIN_MASK)
     int               ex = EX_OK;                /*   */
     mular_descriptor *mular = cfg->qbuf->sun;    /*    */
@@ -232,13 +240,17 @@ srch_sun(BLINr_cfg *cfg, BLIN_trace *twind) {
     last = MULAR_NEXT(mular);
     smem = umem = NULL;
     if  (last == 0) goto put;
-    ifBLIN_QX2("last=%zd", last);
+    ifBLIN_QX2("last=%zu", last);
     while(first != last) {
         mid = (first + last) / 2;
-        ifBLIN_QX4("mid=%zd", mid);
+        ifBLIN_QX4("mid=%zu", mid);
         sun = mular_getix(mular, mid);
-        if  (!sun) isJW(1, "getix(%zd)", mid);
-        ifBLIN_QX4( "getix(%zd) {%u, %s, %u, %s, %u}"
+        if  (!sun) {
+            ifBLIN_QW0("getix(%zu)", mid);
+            ex = 1;
+            goto out;
+        }
+        ifBLIN_QX4( "getix(%zu) {%u, %s, %u, %s, %u}"
                   ,  mid
                   ,  sun->filesz
                   , *sun->file
@@ -258,17 +270,25 @@ srch_sun(BLINr_cfg *cfg, BLIN_trace *twind) {
         }
         if  (abs(com) <= 2) smem = sun->file;
         if  (abs(com) == 1) umem = sun->func;
-        ifBLIN_QX4("first=%zd last=%zd", first, last);
+        ifBLIN_QX4("first=%zu last=%zu", first, last);
     }
 put:
     sun = mular_insert(mular, mid);
-    if  (!sun) isJW(EX_OSERR, "insert %zd", mid);
+    if  (!sun) {
+        ifBLIN_QW0("insert %zu", mid);
+        ex = EX_OSERR;
+        goto out;
+    }
     if  (!smem) {
         ifBLIN_QX2("now strndup file");
         sun->file = malloc(sizeof(char*));
         ifBLIN_QX4("strndup %u %.*s", twind->filesz, twind->filesz, &twind->string[twind->progsz]);
         *sun->file = strndup(&twind->string[twind->progsz], twind->filesz);
-        if  (!sun->file) isJW(EX_OSERR, "file strndup");
+        if  (!sun->file) {
+            ifBLIN_QW0("file strndup");
+            ex = EX_OSERR;
+            goto out;
+        }
     } else {
         sun->file = smem;
     }
@@ -276,73 +296,90 @@ put:
     if  (!umem) {
         ifBLIN_QX2("now strndup func");
         sun->func = malloc(sizeof(char*));
-        ifBLIN_QX4( "strndup %u %.*s"
+        ifBLIN_QX4( "strndup %u %.*s~"
                   , twind->funcsz
                   , twind->funcsz
                   , &twind->string[twind->progsz + twind->filesz]
                   )
         ;
         *sun->func = strndup(&twind->string[twind->progsz + twind->filesz], twind->funcsz);
-        if  (!sun->func) isJW(EX_OSERR, "func strndup");
+        if  (!sun->func) {
+            ifBLIN_QW0("func strndup");
+            ex = EX_OSERR;
+            goto out;
+        }
     } else {
-        ifBLIN_QX4("umem = %"BLIN_U, umem);
+        ifBLIN_QX4("umem = %"BLIN_X, BLIN_I(umem));
         sun->func = umem;
     }
-    ifBLIN_QX4("sun->func = %s", sun->func);
+    ifBLIN_QX4("sun->func = %s~", *sun->func);
     sun->line = twind->line;
     sun->filesz = twind->filesz;
     sun->funcsz = twind->funcsz;
 out:
-    ifBLIN_QX2( "-srch_sun %d %zd {%u, %s, %u, %s, %u}"
+    ifBLIN_QX2( "-srch_sun %d %zu {%u, %s, %u, %s, %u}"
               , com
               , mid
               , sun->filesz
-              , sun->file
+              , *sun->file
               , sun->funcsz
-              , sun->func
+              , *sun->func
               , sun->line
               )
     ;
     return(ex);
-#   undef isJX
-#   undef isJW
 #   undef blin_internal_flags
 }
 
 int
 BLINr_rq(BLINr_cfg *cfg, BLIN_trace *twind) {
-#   define isJX(EXCODE, FORMAT, ...) \
-        do { ifBLIN_QX0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
-#   define isJW(EXCODE, FORMAT, ...) \
-        do { ifBLIN_QW0("!!" FORMAT, ## __VA_ARGS__); ex = EXCODE; goto out; }while(0)
 #   define blin_internal_flags (cfg->flags & BLIN_MASK)
     u_int64_t   fstime;                /*    tracfile         */
     int         ex = EX_OK;            /*                      */
     BLINr_qbuf *qbuf = NULL;           /*                       */
 
     ifBLIN_QX2("now qbuf alloc");
-    if  (!(qbuf = cfg->qbuf = calloc(1, sizeof(BLINr_qbuf)) )) isJW(EX_OSERR, "qbuf alloc");
+    if  (!(qbuf = cfg->qbuf = calloc(1, sizeof(BLINr_qbuf)))) {
+        ifBLIN_QW0("qbuf alloc");
+        ex = EX_OSERR;
+        goto out;
+    }
     qbuf->time1 = qbuf->time2 = fstime = twind->time;
     if  (cfg->flags & (BLINREAD_P_D | BLINREAD_P_T)) {
         ifBLIN_QX4("qbuf->time={%"BLIN_O"u %"BLIN_O"u}", qbuf->time1, qbuf->time2);
     }
     if  (cfg->flags & BLINREAD_P_I) {
         qbuf->pid = mular_create(MULAR_OBJ | MULAR_STRI, 3, sizeof(u_int32_t), msz);
-        if  (!qbuf->pid) isJW(EX_OSERR, "qbuf->pid create");
-    }
+        if  (!qbuf->pid) {
+            ifBLIN_QW0("qbuf->pid create");
+            ex = EX_OSERR;
+            goto out;
+    }   }
     if  (cfg->flags & BLINREAD_P_R) {
         qbuf->prog = mular_create(MULAR_OBJ | MULAR_STRI | MULAR_PFRE, 3, sizeof(char*), msz);
-        if  (!qbuf->prog) isJW(EX_OSERR, "qbuf->prog create");
+        if  (!qbuf->prog) {
+            ifBLIN_QW0("qbuf->prog create");
+            ex = EX_OSERR;
+            goto out;
+        }
         qbuf->prog->tofree = free;    /* free() */
     }
     if  (cfg->flags & (BLINREAD_P_S | BLINREAD_P_U | BLINREAD_P_N)) {
         qbuf->sun = mular_create(MULAR_OBJ | MULAR_STRI, 3, sizeof(BLINr_qbuf_sun), msz);
-        if  (!qbuf->sun) isJW(EX_OSERR, "qbuf->sun create");
+        if  (!qbuf->sun) {
+            ifBLIN_QW0("qbuf->sun create");
+            ex = EX_OSERR;
+            goto out;
+        }
         qbuf->sun->tofree = sunfree;    /* sunfree() */
     }
     for (; ; twind = BLINr_next_wind(cfg)) {
-        if  (!twind) isJW(EX_IOERR, "tracefile window %"BLIN_U, cfg->tfile->ofpoint);
-        ifBLIN_QX4( "twind={%u %u %u %"BLIN_O"u %u %u %u %u %u %.*s}"
+        if  (!twind) {
+            ifBLIN_QW0("tracefile window %"BLIN_O"d", cfg->tfile->ofpoint);
+            ex = EX_IOERR;
+            goto out;
+        }
+        ifBLIN_QX4( "twind={%u %u %d %"BLIN_O"u %d %d %u %u %u %.*s}"
                   , twind->len
                   , twind->level
                   , twind->pid
@@ -374,26 +411,32 @@ BLINr_rq(BLINr_cfg *cfg, BLIN_trace *twind) {
         }
         if  (!!qbuf->pid) {
             if  (!!srch_pid(cfg, twind)) {
-                isJW(EX_OSERR, "qbuf->pid srch");
+                ifBLIN_QW0("qbuf->pid srch");
+                ex = EX_OSERR;
+                goto out;
         }   }
         if  (!!qbuf->prog) {
             if  (!!srch_prog(cfg, twind)) {
-                isJW(EX_OSERR, "qbuf->prog srch");
+                ifBLIN_QW0("qbuf->prog srch");
+                ex = EX_OSERR;
+                goto out;
         }   }
         if  (!!qbuf->sun) {
             if  (!!srch_sun(cfg, twind)) {
-                isJW(EX_OSERR, "qbuf->sun srch");
+                ifBLIN_QW0("qbuf->sun srch");
+                ex = EX_OSERR;
+                goto out;
         }   }
-        if  (mife_ctlsize(cfg->tfile) == twind->len) break;
+        if  (mife_ctlsize(cfg->tfile) == (ssize_t)twind->len) break;
     }
     if  (cfg->flags & BLINREAD_P_D) {
         struct tm *h = NULL;
         time_t     t;
 
-        t = qbuf->time1 / BLINr_E9;
+        t = (time_t)(qbuf->time1 / BLINr_E9);
         h = localtime(&t);
         if  (qbuf->time1 == qbuf->time2) {
-            printf( "datime %u-%02u-%02u %02u:%02u:%02u.%09u\n"
+            printf( "datime %d-%02d-%02d %02d:%02d:%02d.%09u\n"
                   , h->tm_year + 1900
                   , h->tm_mon + 1
                   , h->tm_mday
@@ -403,7 +446,7 @@ BLINr_rq(BLINr_cfg *cfg, BLIN_trace *twind) {
                   , (u_int32_t)(qbuf->time1 % BLINr_E9)
                   );
         } else {
-            printf( "datime from %u-%02u-%02u %02u:%02u:%02u.%09u"
+            printf( "datime from %d-%02d-%02d %02d:%02d:%02d.%09u"
                   , h->tm_year + 1900
                   , h->tm_mon + 1
                   , h->tm_mday
@@ -412,9 +455,9 @@ BLINr_rq(BLINr_cfg *cfg, BLIN_trace *twind) {
                   , h->tm_sec
                   , (u_int32_t)(qbuf->time1 % BLINr_E9)
                   );
-            t = qbuf->time2 / BLINr_E9;
+            t = (time_t)(qbuf->time2 / BLINr_E9);
             h = localtime(&t);
-            printf( " to %u-%02u-%02u %02u:%02u:%02u.%09u\n"
+            printf( " to %d-%02d-%02d %02d:%02d:%02d.%09u\n"
                   , h->tm_year + 1900
                   , h->tm_mon + 1
                   , h->tm_mday
@@ -446,7 +489,11 @@ BLINr_rq(BLINr_cfg *cfg, BLIN_trace *twind) {
 
         printf("\npid:\n");
         for (size_t i = 0; i < MULAR_NEXT(qbuf->pid); i++) {
-            if  (!(mulind = mular_getix(qbuf->pid, i))) isJW(1, "qbuf pid getix");
+            if  (!(mulind = mular_getix(qbuf->pid, i))) {
+                ifBLIN_QW0("qbuf pid getix");
+                ex = 1;
+                goto out;
+            }
             printf("%u ", *(u_int32_t*)mulind);
         }
         printf("\n");
@@ -456,7 +503,11 @@ BLINr_rq(BLINr_cfg *cfg, BLIN_trace *twind) {
 
         printf("\nprog:\n");
         for (size_t i = 0; i < MULAR_NEXT(qbuf->prog); i++) {
-            if  (!(mulind = mular_getix(qbuf->prog, i))) isJW(1, "qbuf prog getix");
+            if  (!(mulind = mular_getix(qbuf->prog, i))) {
+                ifBLIN_QW0("qbuf prog getix");
+                ex = 1;
+                goto out;
+            }
             printf("%s ", *(char**)mulind);
         }
         printf("\n");
@@ -464,11 +515,15 @@ BLINr_rq(BLINr_cfg *cfg, BLIN_trace *twind) {
     if  (cfg->flags & (BLINREAD_P_S | BLINREAD_P_U | BLINREAD_P_N)) {
         BLINr_qbuf_sun *sun, *psun;
 
-        sun = psun = NULL;
+        psun = NULL;
         printf("\n");
         for (size_t i = 0; i < MULAR_NEXT(qbuf->sun); i++) {
             sun = mular_getix(qbuf->sun, i);
-            if  (!sun) isJW(1, "qbuf->sun getix");
+            if  (!sun) {
+                ifBLIN_QW0("qbuf->sun getix");
+                ex = 1;
+                goto out;
+            }
             if  (cfg->flags & BLINREAD_P_S) {
                 if  (  (!psun)
                     || (sun->filesz != psun->filesz)
@@ -502,7 +557,5 @@ BLINr_rq(BLINr_cfg *cfg, BLIN_trace *twind) {
     }   }
 out:
     return(ex);
-#   undef isJX
-#   undef isJW
 #   undef blin_internal_flags
 }
