@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2014..2022 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2014..2024 @BABOLO http://www.babolo.ru/
  * pkg = babolo-libmake
  * All rights reserved.
  *
@@ -25,13 +25,8 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static const char copyright[] = "\
-@(#)Copyright (C)2014..2022 @BABOLO http://www.babolo.ru/\n\
-@(#)All rights reserved.\n\
-@(#)V.M "VERS;
-#endif /* not lint */
-static const char rcsid[] = "$Id: test0.c,v 1.12 2022/04/30 16:49:03 babolo Exp $";
+#ident "@(#) Copyright (C)2014..2024 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: test0.c,v 1.18 2025/04/22 03:13:40 babolo Exp $"
 
 #define BLIN_COMPAT VMAJOR
 
@@ -46,12 +41,12 @@ static const char rcsid[] = "$Id: test0.c,v 1.12 2022/04/30 16:49:03 babolo Exp
 #include <err.h>
 #include "../BLINflag.h"
 
-#define DEF_LEVEL  7
-#define DEF_WIDTH  5
-#define DEF_ACTN   24
-#define DEF_REV    0
-#define DEF_STATEN 8
-#define BASELEN    64
+#define DEF_LEVEL  7U
+#define DEF_WIDTH  5U
+#define DEF_ACTN   24U
+#define DEF_REV    0U
+#define DEF_STATEN 8U
+#define BASELEN    64U
 #define SET(T, X)  do {((u_int64_t*)(T))[(X) / BASELEN] |= (1LLU << ((X) % BASELEN));} while(0)
 
 enum classes
@@ -104,6 +99,7 @@ static struct {
     const char      clasnames[Lx + 1][3];
     const char      statenames[Dx + 1][3];
     const char      actnames[24][3];
+    const char      dummy[3];
     const u_int32_t automa[Dx][Lx];
 } z
 #if 0
@@ -157,13 +153,16 @@ static struct {
   , { Ce|Dx,    Dg,    Dg,    Dg,    Dq,          Dg,    Dg,    Dg,    Dg,    Dg,    Dg,    Dg}/*Dg*/
   , { Ce|Dx, Ce|Dx, C2|Dx, C2|De, Ce|Dx,    Co|Cp|Dv, Ch|Dv, Ce|Dx, Co|Dv, Ce|Dx, C2|Dd, Ce|Dx}/*Do*/
   , { Ce|Dx, Ce|Dx, C2|Dx, C2|De, Ce|Dx,       Cp|Dv, Ce|Dx, Cb|Dv, Cp|Dv, Ce|Dx, C2|Dd, Ce|Dx}/*Dv*/
-} };
+  }
+, { 0, 0, 0 }
+};
 #else
 ;
 #endif
 
 // char q[] = "( 0x34 056 56 [dfg] 0xfd..567)";
 
+__attribute__((__noreturn__))
 static void
 usage(int ex) {
     fprintf( stderr
@@ -188,14 +187,22 @@ usage(int ex) {
 
 int
 main(int argc, char **argv) {
-    u_int32_t r0 = 0, r1 = 0, d = 0, control = 0, flags = BLIN_BIT0;
-    u_int64_t conf[65536 / BASELEN];
-    ssize_t i, j, shift = 0;
-    int fd, c, ex = EX_OK;
-    char *p, *q = NULL;
-    enum states state;
-    blin_statectl sc;
-    const char *cn;
+    u_int32_t      control = 0;
+    u_int32_t      flags   = BLIN_BIT0;
+    enum states    state;
+    ssize_t        shift   = 0;
+    const char    *flar    = "a:F:f:I:i:l:qrS:s:vw:=:?";
+    u_int64_t      conf    [65536 / BASELEN];
+    int            ex      = EX_OK;
+    blin_statectl  sc;
+    int            fd;
+    u_int32_t      r0      = 0;
+    u_int32_t      r1      = 0;
+    u_int32_t      d       = 0;
+    char          *q       = NULL;
+    char          *p;
+    ssize_t        i;
+    int            c;
 
     blin_ctl(BLIN_CTL_FLAG | BLIN_CTL_FSET, flags);
     blin_ctl(BLIN_CTL_WHAT | BLIN_CTL_FSET | BLIN_BIT1, BLIN_MODTIME);
@@ -205,10 +212,10 @@ main(int argc, char **argv) {
     blin_ctl(BLIN_CTL_WHAT | BLIN_CTL_FSET | BLIN_BIT7, BLIN_MODDREL | BLIN_MODDSTR);
     blin_ctl(BLIN_CTL_DUMP);
     ifBLIN_QX0("begin");
-    sc = (blin_statectl){{DEF_LEVEL, 1, DEF_STATEN, DEF_ACTN, ' ', DEF_WIDTH, DEF_REV}};
-    while ((c = getopt(argc, argv, "a:F:f:I:i:l:qrS:s:vw:?")) != -1) {
+    sc = (blin_statectl){{DEF_LEVEL, 1, DEF_STATEN, DEF_ACTN, ' ', DEF_WIDTH, DEF_REV, 0, 0}};
+    while ((c = getopt(argc, argv, flar)) != -1) {
         switch (c) {
-        case 'a': sc.actinn = strtoul(optarg, NULL, 0);
+        case 'a': sc.actinn = (u_int32_t)strtoul(optarg, NULL, 0);
                 ; break;
         case 'F': if  (0 > (fd = open(optarg, O_WRONLY | O_CREAT | O_TRUNC | O_NOFOLLOW, 0666))) {
                       ifBLIN_QW0("open for write");
@@ -237,27 +244,32 @@ main(int argc, char **argv) {
                 ; break;
         case 'i': q = optarg;
                 ; break;
-        case 'l': sc.level = strtoul(optarg, NULL, 0);
+        case 'l': sc.level = (u_int32_t)strtoul(optarg, NULL, 0);
                 ; break;
         case 'q': BLIN_QUIET(flags);
                 ; break;
         case 'r': sc.revert = ~sc.revert;
                 ; break;
-        case 'S': shift = strtoul(optarg, NULL, 0);
+        case 'S': shift = (ssize_t)strtoul(optarg, NULL, 0);
                 ; break;
-        case 's': sc.staten = strtoul(optarg, NULL, 0);
+        case 's': sc.staten = (u_int32_t)strtoul(optarg, NULL, 0);
                 ; break;
         case 'v': BLIN_VERBOSE(flags);
                 ; break;
-        case 'w': sc.allign = strtoul(optarg, NULL, 0);
+        case 'w': sc.allign = (u_int32_t)strtoul(optarg, NULL, 0);
+                ; break;
+        case '=': if  (!(p = optarg)) usage(EX_USAGE);
+                ; switch(p[0]) {
+                  case '0': blin_ctl(BLIN_CTL_DUMP);      exit(EX_OK);
+                  case 'T': blin_ctl(BLIN_CTL_TCFN, ++p); break;
+                  default:                                usage(EX_USAGE);
+                ; }
                 ; break;
         case '?': usage(EX_OK);
         default : ifBLIN_QX0("Illegal flag");
                 ; usage(EX_USAGE);
     }   }
-    argc -= optind;
-    argv += optind;
-    ifBLIN_QX0( "blin_statectl %08X <l=%u s=%u a=%u w=%u r=%u>"
+    ifBLIN_QX0( "blin_statectl %08X <l=%d s=%u a=%u w=%u r=%u>"
               , sc.statectl
               , sc.level
               , sc.staten
@@ -265,12 +277,24 @@ main(int argc, char **argv) {
               , sc.allign
               , sc.revert
               );
-    ifBLIN_QX0("%"BLIN_X" %"BLIN_X, blin_internal_flags, flags);
+    ifBLIN_QX0("%08X %08X", blin_internal_flags, flags);
     blin_stateheader(sc, z.actnames);
     for (state = Di, p = q; q && (state < Dx); ++p) {
-        control = z.automa[state][z.recode[*p]];
-        cn = z.clasnames[z.recode[*p]];
-        blin_statebody(sc, z.actnames, z.statenames, cn, p, p - q, control, state, r0, d);
+        const char    *cn;
+
+        control = z.automa[state][z.recode[*p & 0x0FF]];
+        cn = z.clasnames[z.recode[*p & 0x0FF]];
+        blin_statebody( sc
+                      , z.actnames
+                      , z.statenames
+                      , cn
+                      , p
+                      , (size_t)(p - q)
+                      , control
+                      , state
+                      , r0
+                      , (u_char)(d & 0xFF)
+                      );
         if  (C1 & control) r1 = r0;
         if  (C0 & control) SET((void*)&conf, r1);
         if  ((Cz | C0 | C1) & control) r0 = 0;
@@ -278,31 +302,31 @@ main(int argc, char **argv) {
         if  (Co & control) d = 8;
         if  (Ch & control) d = 16;
         if  (Cp & control) {
-            u_int32_t i;
+            u_int32_t k;
 
-            i = *p - '0';
-            if  (i > d) control = Ce | Dx; else r0 = r0 * d + i;
+            k = (u_int32_t)(*p - '0');
+            if  (k > d) control = Ce | Dx; else r0 = r0 * d + k;
         }
         if  (Cb & control) {
             if  (16 < d) {
                 control = Ce | Dx;
             } else {
-                u_int32_t i;
+                u_int32_t k;
 
-                i = *p - 'A';
-                if  (i > 6) i -= 'a' - 'A';
-                r0 = r0 * d + i + 10;
+                k = (u_int32_t)(*p - 'A');
+                if  (k > 6) k -= 'a' - 'A';
+                r0 = r0 * d + k + 10;
         }   }
         if  (C2 & control) {
-            u_int32_t i;
+            u_int32_t k;
 
-            for (i = r1; i < r0; i++) SET((void*)&conf, i);
+            for (k = r1; k < r0; k++) SET((void*)&conf, k);
         }
         if  (Ce & control) warnx("bsdfig: c_lists: syntax");
         state = control & C_state;
     }
-    blin_dumb(BLIN_MODDTXT | sc.level, (char*)(void*)&z + shift, sizeof(z) - shift);
-    blin_dumb(7, (char*)(void*)&z + shift, sizeof(z) - shift, "===>>> ");
+    blin_dumb(BLIN_MODDTXT | sc.level, (char*)(void*)&z + shift, (ssize_t)sizeof(z) - shift);
+    blin_dumb(7, (char*)(void*)&z + shift, (ssize_t)sizeof(z) - shift, "===>>> ");
     errno = 1981284353;
     ifBLIN_QW1("test %d", errno);
     errno = -1981284353;
